/*
  Copyright (c) 2012, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "ffd.h"
#include "neigh.h"


static ffd_nexthop_t* select_nexthop(const ffd_announce_t *announce) {
	uint16_t ret_metric = 0xffff;
	ffd_nexthop_t *ret = NULL;

	ffd_nexthop_t *nexthop;
	for (nexthop = announce->nexthop_list; nexthop; nexthop = nexthop->next) {
		if (!ffd_is_feasible(announce, nexthop->metric_seqno))
			continue;

		uint32_t metric = nexthop->metric_seqno.metric + ffd_neigh_get_cost(nexthop->neigh);

		if (metric < ret_metric) {
			ret = nexthop;
			ret_metric = metric;
		}
	}

	return ret;
}

ffd_metric_seqno_t get_metric(const ffd_announce_t *announce) {
	if (announce->selected) {
		uint32_t metric = announce->selected->metric_seqno.metric + ffd_neigh_get_cost(announce->selected->neigh);

		if (metric < 0xffff)
			return (ffd_metric_seqno_t){metric, announce->selected->metric_seqno.seqno};
	}

	return (ffd_metric_seqno_t){0xffff, 0};
}

static inline void update_selected(ffd_announce_t *announce) {
	announce->selected = select_nexthop(announce);
	announce->metric = get_metric(announce);
}

void ffd_announce_update(ffd_announce_t *announce, ffd_nexthop_t *nexthop, ffd_metric_seqno_t ms, uint16_t interval) {
	nexthop->metric_seqno = ms;
	nexthop->interval = interval;

	if (ms.metric != 0xffff)
		nexthop->last_update = now;

	update_selected(announce);
}

ffd_announce_t* ffd_announce_new(void) {
	ffd_announce_t *a = calloc(1, sizeof(ffd_announce_t));

	a->metric.metric = a->feasibility_distance.metric = 0xffff;

	a->next = announce_list;
	announce_list = a;

	return a;
}
